@@ -2,60 +2,69 @@ import io
 import json
 import logging
 import os
-from typing import TYPE_CHECKING, Dict, Iterator, Union
+from typing import TYPE_CHECKING, Any, Dict, Iterator, Optional, Union
 
 import requests
 from requests.adapters import HTTPAdapter
+from urllib3 import Retry
 
-from odp.tabular_v2.client.client import read_retry_strategy, write_retry_strategy
+from .util.check_version import get_sdk_version
 
 # NOTE(oha): deferred imports to avoid circular dependencies, will be fixed when we remove the old client
 # from odp.client.auth import InteractiveTokenProvider
-# from odp.client.catalog_client import CatalogClient
-from odp.tabular_v2.util import Iter2Reader
 
 if TYPE_CHECKING:
-    from dataset import Dataset
+    from .dataset import Dataset
+
+read_retry_strategy = Retry(
+    total=8,
+    status_forcelist=[429, 502, 503, 504],
+    allowed_methods=["HEAD", "GET", "OPTIONS", "POST"],
+    backoff_factor=1,  # exponential sleep (1s, 2s, 4s…)
+)
+
+write_retry_strategy = Retry(
+    total=8,
+    status_forcelist=[429],
+    allowed_methods=["HEAD", "GET", "OPTIONS", "POST"],
+    backoff_factor=2,
+)
 
 
-class _Response:
-    # Abstraction for the response object, shared between http client and test client
-    def __init__(self, res: Union[requests.Response, Iterator[bytes], Dict, bytes]):
-        logging.info("response: %s", res)
-        if isinstance(res, requests.Response):
-            if res.status_code == 204:
-                raise FileNotFoundError(res.text)
-            res.raise_for_status()
-        # logging.info("response: %s", res)
+class Response:
+    def __init__(self, res: requests.Response):
+        if res.status_code == 204:
+            raise FileNotFoundError(res.text)
+        if res.status_code == 400:
+            raise ValueError(res.text)
+        res.raise_for_status()
         self.res = res
 
     def reader(self):
-        if isinstance(self.res, bytes):
-            return io.BytesIO(self.res)
-        if isinstance(self.res, Iterator):
-            return Iter2Reader(self.res)
+        if self.res.raw is None:
+            return io.BytesIO(self.res.content)
         return self.res.raw
 
     def iter(self) -> Iterator[bytes]:
-        if isinstance(self.res, bytes):
-            return iter([self.res])
-        if isinstance(self.res, Iterator):
-            return self.res
-        return self.res.iter_content()
+        if self.res.raw is None:
+            return iter([self.res.content])
+        return self.res.raw
 
     def all(self) -> bytes:
-        if isinstance(self.res, bytes):
-            return self.res
-        if isinstance(self.res, Iterator):
-            return b"".join(self.res)
+        if self.res.raw is None:
+            return self.res.content
+        return self.res.raw.read()
+
+    def json(self) -> Union[dict, list, None]:
+        return self.res.json()
+
+    @property
+    def content(self) -> Union[bytes, Any]:
         return self.res.content
 
-    def json(self) -> dict:
-        if self.res is None:
-            return None
-        if isinstance(self.res, dict):
-            return self.res
-        return self.res.json()
+    @property
+    def status_code(self) -> int:
+        return self.res.status_code
 
 
 class Client:
@@ -71,6 +80,7 @@ class Client:
             if not jwt_bearer.startswith("Bearer "):
                 jwt_bearer = "Bearer " + jwt_bearer
             self._auth = lambda: jwt_bearer
+
         elif os.getenv("JUPYTERHUB_API_TOKEN"):
 
             def get_token():
@@ -88,9 +98,6 @@ class Client:
         # FIXME: this is wrong, sessions can't be this long
         self._http_client = requests.Session()
         self._http_client.headers.setdefault("User-Agent", "odp-sdk-python")
-        from odp.client.catalog_client import CatalogClient
-
-        self._catalog = CatalogClient(http_client=self._http_client)
 
     def _setup_jwt(self):
         """
@@ -103,13 +110,29 @@ class Client:
         self._auth = lambda: f"Bearer {prov.get_token()}"
         # raise NotImplementedError("JWT authentication is not implemented yet.")
 
+    def request(
+        self,
+        path: str,
+        method: str = "POST",
+        params: Optional[dict] = None,
+        data: Union[Dict, bytes, Iterator[bytes], io.IOBase, None] = None,
+        retry: bool = False,
+    ) -> Response:
+        """
+        send a request to ODP
+        """
+        req = requests.Request(method, self.base_url + path, params=params, data=data)
+        res = self._request(req, retry=retry)
+
+        return Response(res)
+
     def _request(self, req: requests.Request, retry: bool = True) -> requests.Response:
         """
         base implementation to send a request to ODP
         may be overridden to use mechanisms other than http
         """
         req.headers.setdefault("Authorization", self._auth())  # allow override
-        req.headers.setdefault("User-Agent", "odp-sdk-python")
+        req.headers.setdefault("User-Agent", "odp-sdk-python/" + get_sdk_version())
         if isinstance(req.data, dict):
             req.data = json.dumps(req.data)
             req.headers.setdefault("Content-Type", "application/json")
@@ -133,12 +156,4 @@ class Client:
         """fetch a dataset by id"""
         from .dataset import Dataset
 
-        # NOTE(oha): ktable does not strictly need a catalog, and catalog v2 is not ready yet
-        # we just silently pass a dataset with not catalog features for now, and add more later
         return Dataset(self, id)
-        # req = requests.Request("GET", self.base_url + "/api/catalog/v2/datasets/" + safe_id(id))
-        # req.timeout = 10
-        # res = self._request(req)
-        # if res.status_code in (204, 404):
-        #    raise FileNotFoundError(f"Dataset {id} not found: %s %s", res.status_code, res.content)
-        # return Dataset._from_catalog_v2(self, res.json())
