@@ -2,86 +2,59 @@ import io
 import json
 import logging
 import typing
-from functools import partial
 from typing import TYPE_CHECKING, Dict, Iterator, List, Optional, Union
 
 import pandas as pd
 import pyarrow as pa
-import requests
 
-import odp.tabular_v2.client as old
 from odp.tabular_v2.client.table_raw import Raw
+from odp.tabular_v2.client.table_stats import TableStats
 from odp.tabular_v2.util import vars_to_json
-from odp.util.cheapdantic import BaseModel
 
 if TYPE_CHECKING:
+    import odp.new_client as new
     from odp.tabular_v2.client import Cursor
 
 
-class TableStats(BaseModel):
-    num_rows: int
-    """number of rows in the table"""
-
-    size: int
-    """size of the table in bytes, including metadata and schema"""
-
-
-# temporary: migrate old API to the new client
-def _request_to_new_client(
-    cli,  # new client instance
-    path: str,
-    data: Union[Dict, bytes, Iterator[bytes], io.IOBase, None] = None,
-    params: Optional[Dict] = None,
-    headers: Optional[Dict] = None,
-    retry: bool = True,  # if it is safe to retry the request on failure
-) -> old.Client.Response:
-    req = requests.Request(
-        method="POST",
-        url=cli.base_url + path,
-        params=params,
-        headers=headers,
-        data=data or b"",
-    )
-    res = cli._request(req, retry)
-    return old.Client.Response(res)
-
-
 class Table:
-    def __init__(self, impl, table_id: str):
+    def __init__(self, cli: "new.Client", table_id: str):
         self._id = table_id
         self.max_insert_size = 50_000_000  # maximum number of bytes for a single insert chunk
         self.max_insert_time = 30.0  # min time before flushing on a new request
-
-        # unify new and old client _request() method
-        import odp.new_client as new
-
-        if isinstance(impl, new.Client):
-            self._impl = partial(_request_to_new_client, impl)
-        elif isinstance(impl, old.Client):
-            self._impl = impl._request
-        else:
-            raise ValueError(f"Unexpected implementation type: {type(impl)}")
-
+        self.cli: "new.Client" = cli
         self._tx = None
 
     @property
     def raw(self) -> "Raw":
         return Raw(self)
 
+    def analyze(self):
+        res = self.cli.request(
+            path="/api/table/v2/analyze",
+            params={
+                "table_id": self._id,
+            },
+        )
+        return res.json()
+
     def stats(self) -> TableStats:
         """
         Get the table statistics, such as the number of rows, size, etc.
         @return: a TableStats object containing the statistics, including:
                  - num_rows: the number of rows in the table
                  - size: the size of the table in bytes, including metadata and schema
+                 - columns: a list of the columns with their statistics
         """
-        res = self._impl(
+        res = self.cli.request(
             path="/api/table/v2/stats",
             params={
                 "table_id": self._id,
             },
         )
-        return TableStats(**res.json())
+        j = res.json()
+        logging.warning("table %s stats: %s", self._id, json.dumps(j, indent=2))
+        stats = TableStats(_table=self, **j)
+        return stats
 
     def schema(self) -> typing.Optional[pa.Schema]:
         try:
@@ -96,7 +69,7 @@ class Table:
         """
         Remove all data from the table, but keep the schema.
         """
-        self._impl(
+        self.cli.request(
             path="/api/table/v2/truncate",
             params={
                 "table_id": self._id,
@@ -120,7 +93,7 @@ class Table:
         )
         w.close()
 
-        res = self._impl(
+        res = self.cli.request(
             path="/api/table/v2/sdk/alter",
             params={
                 "table_id": self._id,
@@ -137,7 +110,7 @@ class Table:
         @return:
         """
         try:
-            res = self._impl(
+            res = self.cli.request(
                 path="/api/table/v2/drop",
                 params={
                     "table_id": self._id,
@@ -180,7 +153,7 @@ class Table:
         w.write_batch(pa.RecordBatch.from_pylist([], schema=schema))
         w.close()
 
-        self._impl(
+        self.cli.request(
             path="/api/table/v2/sdk/create",
             params={
                 "table_id": self._id,
@@ -342,7 +315,7 @@ class Table:
         if self._tx:
             raise ValueError("already in a transaction")
 
-        res = self._impl(
+        res = self.cli.request(
             path="/api/table/v2/begin",
             params={
                 "table_id": self._id,
@@ -358,7 +331,7 @@ class Table:
         if exc_type is not None:
             logging.warning("aborting transaction %s", self._tx._id)
             # try:
-            #    self._impl(
+            #    self.cli.request(
             #        path="/api/table/v2/rollback",
             #        params={
             #            "table_id": self._id,
@@ -369,7 +342,7 @@ class Table:
             #    logging.error("ignored: rollback failed: %s", e)
         else:
             self._tx.flush()
-            self._impl(
+            self.cli.request(
                 path="/api/table/v2/commit",
                 params={
                     "table_id": self._id,
@@ -394,7 +367,7 @@ class Table:
     ) -> Iterator[pa.RecordBatch]:
         # t0 = time.perf_counter()
         while cursor is not None:
-            res = self._impl(
+            res = self.cli.request(
                 path="/api/table/v2/sdk/" + typ,
                 params={
                     "table_id": self._id,
@@ -439,7 +412,7 @@ class Table:
         w.write_batch(data)
         w.close()
 
-        self._impl(
+        self.cli.request(
             path="/api/table/v2/sdk/insert",
             params={
                 "table_id": self._id,
